
include quartz.e
include graphics.e
include qvertex.e
include mod_sort.e

 ----------

constant NIL = 0

global constant ASCENDING   = 0
global constant DESCENDING  = 1

sequence DEFAULT_TRIANGLE, ZVALUE_ENTRY
DEFAULT_TRIANGLE = {{0,0,0}, {0,0,0}, {0,0,0}}
ZVALUE_ENTRY = {0.0, {}}

 ----------

function Abs (atom x)
   if (x < 0) then
      return -x
   else
      return x
   end if
end function -- end Abs()

 ----------

 -- This function is provided so any set of polygons can be sorted
 -- based on their average z-depth; for drawing, it works fairly well.
 -- It's not always accurate, but for simple shapes or situations
 -- with a lot of motion it works fine. Also, sort second on the
 -- y-distance from the origin; this way, those further from directly
 -- in front of you are drawn first; and sort of x-distance last.
global function ZSort (sequence Polygons, integer Direction)

   sequence Vertices, ZValues, ZValue
   integer  PLen, VLen
   atom     z --, y, x
   qobject  CurrPolygon --, Vertex

   PLen = length (Polygons)
   ZValues = repeat (ZVALUE_ENTRY, PLen)
   ZValue = ZVALUE_ENTRY
   for i = 1 to PLen do
      CurrPolygon = Polygons[i]
      Vertices = get_qmember (CurrPolygon, "Vertices")
      VLen = length (Vertices)
      --x = 0
      --y = 0
      z = 0
      for j = 1 to VLen do
         --Vertex = Vertices[j]
         --x = x + get_qmember (Vertex, "x")
         --y = y + get_qmember (Vertex, "y")
         z = z + get_qmember (Vertices[j], "z")
      end for
      ZValue[1] = z --, Abs (y/VLen), Abs (x/VLen),
      ZValue[2] = CurrPolygon
      ZValues[i] = ZValue
   end for

   if (Direction = ASCENDING) then
      ZValues = sort (ZValues)
   else
      ZValues = dsort (ZValues) -- sort in descending order
   end if

   for i = 1 to PLen do
      Polygons[i] = ZValues[i][2]
   end for

   return Polygons

end function -- end ZSort()

 ----------

without warning

 -- Param is the color, followed by a three x-y-z coordinates, each of
 -- which is a simple sequence, NOT a q-object
global function Polygon_Polygon (qobject Self, object Param)

   Self = set_qmember (Self, "Color",   Param[1])
   Self = set_qmember (Self, "Vertices",
                       {new_qobject ("Vertex", Param[2]),
                        new_qobject ("Vertex", Param[3]),
                        new_qobject ("Vertex", Param[4])})

   return Self

end function -- end Polygon_Polygon()


global function Polygon_DrawSelf (qobject Self, object Param)

   sequence points, Vertices

    -- first, let's translate the 3D vertices to their 2D points
   Vertices = get_qmember (Self, "Vertices")
   points = DEFAULT_TRIANGLE
   points[1] = send_qmessage (Vertices[1], "Translate", 0)
   points[2] = send_qmessage (Vertices[2], "Translate", 0)
   points[3] = send_qmessage (Vertices[3], "Translate", 0)

    -- now that the shifts are done, we draw the polygon in 2D
   polygon (get_qmember (Self, "Color"), 1, points)

   return NIL

end function -- end Polygon_DrawSelf()


global function Polygon_MoveX (qobject Self, object Param)

   sequence Vertices

   Vertices = get_qmember (Self, "Vertices")

   for i = 1 to 3 do
      Vertices[i] = send_qmessage (Vertices[i], "MoveX", Param)
   end for

   Self = set_qmember (Self, "Vertices", Vertices)

   return Self

end function -- end Polygon_MoveX()


global function Polygon_MoveY (qobject Self, object Param)

   sequence Vertices

   Vertices = get_qmember (Self, "Vertices")

   for i = 1 to 3 do
      Vertices[i] = send_qmessage (Vertices[i], "MoveY", Param)
   end for

   Self = set_qmember (Self, "Vertices", Vertices)

   return Self

end function -- end Polygon_MoveY()


global function Polygon_MoveZ (qobject Self, object Param)

   sequence Vertices

   Vertices = get_qmember (Self, "Vertices")

   for i = 1 to 3 do
      Vertices[i] = send_qmessage (Vertices[i], "MoveZ", Param)
   end for

   Self = set_qmember (Self, "Vertices", Vertices)

   return Self

end function -- end Polygon_MoveZ()


 -- like the Rotate method for a Vertex, this rotates around a
 -- specified central point included with the degree specifications;
 -- if there's no specified center, the origin is assumed.
global function Polygon_Rotate (qobject Self, object Param)

   sequence Vertices

   Vertices = get_qmember (Self, "Vertices")

   for i = 1 to 3 do
      Vertices[i] = send_qmessage (Vertices[i], "Rotate", Param)
   end for

   Self = set_qmember (Self, "Vertices", Vertices)

   return Self

end function -- end Polygon_Rotate()


 -- like the Scalde method for a Vertex, this scales the polygon
 -- relative to the origin (meaning, unless manually centered,
 -- translation (movement) may take place simultaneously)
global function Polygon_Scale (qobject Self, object Param)

   sequence Vertices

   Vertices = get_qmember (Self, "Vertices")

   for i = 1 to 3 do
      Vertices[i] = send_qmessage (Vertices[i], "Scale", Param)
   end for

   Self = set_qmember (Self, "Vertices", Vertices)

   return Self

end function -- end Polygon_Scale()

with warning

 ----------

define_qclass ("Polygon", BASE_QCLASS,
               {"Color", "Vertices"},
               {"DrawSelf", "MoveX", "MoveY", "MoveZ", "Rotate",
                "Scale"},
               {routine_id ("Polygon_DrawSelf"),
                routine_id ("Polygon_MoveX"),
                routine_id ("Polygon_MoveY"),
                routine_id ("Polygon_MoveZ"),
                routine_id ("Polygon_Rotate"),
                routine_id ("Polygon_Scale")},
               routine_id ("Polygon_Polygon"))

